@@ -13,7 +13,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
- 
+
+#define _FILE_OFFSET_BITS  64
+#include <config.h>
 #include <gnome.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <fcntl.h>
@@ -22,6 +24,9 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <string.h>
 #include "gwget_data.h"
@@ -29,12 +34,14 @@
 #include "main_window.h"
 #include "main_window_cb.h"
 #include "utils.h"
+#include "systray.h"
+
 
-#include <config.h>
 
 static gint gwget_data_process_information (GwgetData *gwgetdata);
 static void gwget_data_update_statistics_ui(GwgetData *gwgetdata);
 static void convert_time2str(gchar *buffer, guint32 time);
+static void gwget_gnotify_finished(GwgetData *gwgetdata);
 
 static void
 convert_time2str (gchar *buffer, guint32 time)
@@ -50,16 +57,17 @@ gwget_data_set_state (GwgetData *gwgetdata, DlState state)
 {
 	gwgetdata->state=state;
 	gwget_data_update_statistics_ui(gwgetdata);
+	gwget_remember_downloads();
 }
 
 void 
 gwget_data_update_statistics (GwgetData *gwgetdata) 
 {   
-	guint32 cur_size;
+	guint64 cur_size;
 	time_t cur_time, elapsed_time;
 	struct stat file_stat;
 	gchar buffer[20];
-	guint32 retr_size, remain_size;
+	guint64 retr_size, remain_size;
 	time_t estimated;
 	gdouble perc;
 	gchar *title;
@@ -67,7 +75,7 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 	
 	/* Get time and size of the file being retrieved */
 	if (stat (gwgetdata->local_filename, &file_stat) != -1) {
-		cur_size = file_stat.st_size;
+		cur_size = (guint64) file_stat.st_size;
 		cur_time = file_stat.st_ctime;
 	} else {
 		cur_size = 0;
@@ -80,9 +88,13 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 				-1);
 	
 	/* Set the total and session time */
-	if ( (gwgetdata->state != DL_NOT_STARTED) && 
-	     (gwgetdata->state != DL_NOT_RUNNING) && 
-	     (gwgetdata->state != DL_COMPLETED)) {
+	if ((gwgetdata->state != DL_NOT_STARTED) && 			
+            (gwgetdata->state != DL_NOT_RUNNING) &&
+	    (gwgetdata->state != DL_NOT_CONNECTED) &&
+	    (gwgetdata->state != DL_ERROR) &&
+	    (gwgetdata->state != DL_WAITING) &&
+	    (gwgetdata->state != DL_COMPLETED))
+	{
 		elapsed_time = cur_time - gwgetdata->session_start_time;
 		gwgetdata->total_time += elapsed_time - gwgetdata->session_elapsed;
 		gwgetdata->session_elapsed = elapsed_time;
@@ -108,7 +120,7 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 	if (gwgetdata->state == DL_NOT_STARTED)
 		strcpy (buffer, "");
 	else
-		sprintf (buffer, "%d kB", (gwgetdata->total_size + 512) / 1024);
+		sprintf (buffer, "%d kB", (guint32)(gwgetdata->total_size + 512) / 1024);
 	
 	gtk_list_store_set(GTK_LIST_STORE(model),&gwgetdata->file_list,
 						TOTALSIZE_COLUMN,buffer,
@@ -119,36 +131,37 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 	if (gwgetdata->state == DL_NOT_STARTED || gwgetdata->state == DL_COMPLETED)
 		strcpy (buffer, "");
 	else
-		sprintf (buffer, "%d kB", (gwgetdata->cur_size + 512) / 1024);
+		sprintf (buffer, "%d kB",(guint32) (gwgetdata->cur_size + 512) / 1024);
 	gtk_list_store_set(GTK_LIST_STORE(model),&gwgetdata->file_list,
 						CURRENTSIZE_COLUMN,buffer,
 						-1);
 	
 	
 	/* Update estimated and remain times */
-	if ((gwgetdata->state != DL_NOT_STARTED) &&
-            (gwgetdata->state != DL_NOT_RUNNING) && 
-            (gwgetdata->state != DL_COMPLETED)) {
+	if ((gwgetdata->state != DL_NOT_STARTED) && 			
+            (gwgetdata->state != DL_NOT_RUNNING) &&
+	    (gwgetdata->state != DL_NOT_CONNECTED) &&
+	    (gwgetdata->state != DL_ERROR) &&
+	    (gwgetdata->state != DL_WAITING) &&
+	    (gwgetdata->state != DL_COMPLETED)) 
+	{	
         	if (gwgetdata->total_size != 0 && retr_size != 0) { 
 			estimated = (((gfloat) (gwgetdata->total_size - gwgetdata->session_start_size)
                   			* gwgetdata->session_elapsed) / retr_size)
                 			+ ((gfloat) (gwgetdata->total_time
                              		- gwgetdata->session_elapsed)); 
 		} else {
-            		estimated = 0;
+			estimated=0;
 		}
 	} else {
-        	if ((gwgetdata->total_size != 0) && (gwgetdata->cur_size != 0) &&
-            		(gwgetdata->state != DL_COMPLETED)) {
-			estimated = ((gfloat) gwgetdata->total_size * gwgetdata->total_time) / gwgetdata->cur_size;
-		} else {
-            		estimated = 0;
-		}
+		estimated = 0;
 	}
+
 	if (estimated == 0)
 		strcpy (buffer, "");
 	else
 		convert_time2str (buffer, estimated);
+
 	
 	gtk_list_store_set(GTK_LIST_STORE(model),&gwgetdata->file_list,
 						ESTIMATEDTIME_COLUMN,buffer,
@@ -170,21 +183,24 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 			PERCENTAGE_COLUMN,(gint)perc,-1);
 	title = g_strdup_printf("%s %d %%", gwgetdata->filename, (gint)perc);
 	/* Speed column */
-	if ((gwgetdata->state != DL_NOT_STARTED) &&
-		(gwgetdata->state != DL_NOT_RUNNING) &&
-		(gwgetdata->state != DL_COMPLETED)) {
-        		if (gwgetdata->session_elapsed != 0) {
-            			if (retr_size == 0) {
-					strcpy (buffer, _ ("stalled"));
-				} else {
-                			sprintf (buffer, "%.2f kB/s",
-                         			((gfloat) retr_size / gwgetdata->session_elapsed) / 1024);
-				}
+	if ((gwgetdata->state != DL_NOT_STARTED) && 			
+            (gwgetdata->state != DL_NOT_RUNNING) &&
+	    (gwgetdata->state != DL_NOT_CONNECTED) &&
+	    (gwgetdata->state != DL_ERROR) &&
+	    (gwgetdata->state != DL_WAITING) &&
+	    (gwgetdata->state != DL_COMPLETED) &&
+	    (gwgetdata->session_elapsed != 0))
+	{
+            		if (retr_size == 0) {
+				strcpy (buffer, _("stalled"));
+			} else {
+                		sprintf (buffer, "%.2f kB/s",
+                        			((gfloat) retr_size / gwgetdata->session_elapsed) / 1024);
 			}
 	} else {
 		strcpy (buffer, "");
 	}
-	
+
 	gtk_list_store_set(GTK_LIST_STORE(model),&gwgetdata->file_list,
 			   SPEED_COLUMN,buffer,-1);
 	
@@ -230,6 +246,7 @@ gwget_data_update_statistics_ui(GwgetData *gwgetdata)
 							   gwgetdata->url);
 					   gtk_tooltips_set_tip (GTK_TOOLTIPS(tray_tooltip), GTK_WIDGET(tray_icon),tooltip_message,NULL);
 					   egg_tray_icon_send_message(tray_icon,100000,tooltip_message,strlen(tooltip_message));
+					   set_icon_idle();
 				   }
 					   
 				   break;
@@ -273,6 +290,7 @@ gwget_data_process_information (GwgetData *gwgetdata)
 		else if (WIFEXITED (status)) {
             		if (WEXITSTATUS (status) == 0) {
                 		gwget_data_set_state (gwgetdata, DL_COMPLETED);
+                     gwget_gnotify_finished(gwgetdata);
 				start_first_waiting_download();
 				if (gwget_pref.open_after_dl) {
 					gwget_data_exec(gwgetdata);
@@ -421,6 +439,7 @@ gwget_data_start_download(GwgetData *gwgetdata)
 			     gwget_pref.http_proxy ) 
 			{
 				setenv("http_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
+				setenv("ftp_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
 		 	        argv[arg]="-Yon";
 				arg++;
 			}
@@ -428,6 +447,7 @@ gwget_data_start_download(GwgetData *gwgetdata)
 			if ( strcmp (gwget_pref.network_mode, "default" ) == 0 && gwget_pref.gnome_http_proxy && gwget_pref.gnome_use_proxy) 
 			{
 				setenv("http_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.gnome_http_proxy),":",g_strdup_printf("%d",gwget_pref.gnome_http_proxy_port),NULL),1);
+				setenv("ftp_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
 	            		argv[arg]="-Yon";
 				arg++;
 			}
@@ -470,7 +490,7 @@ gwget_data_start_download(GwgetData *gwgetdata)
 }
 
 void
-gwget_data_set_total_size (GwgetData *gwgetdata,guint32 total_size)
+gwget_data_set_total_size (GwgetData *gwgetdata, guint64 total_size)
 {
 	g_return_if_fail (gwgetdata != NULL);
 	gwgetdata->total_size = total_size;
@@ -517,7 +537,7 @@ gwget_data_create(gchar *url, gchar *dir)
 		gnome_vfs_get_file_info (gwgetdata->local_filename, 
 				         info, 
 					 GNOME_VFS_FILE_INFO_DEFAULT);
-		gwgetdata->cur_size = (guint32)info->size;
+		gwgetdata->cur_size = (guint64)info->size;
 	} else {
 		gwgetdata->cur_size = 0;
 	}
@@ -546,6 +566,7 @@ gwget_data_set_filename(GwgetData* gwgetdata,gchar *str)
 	gwgetdata->local_filename = g_strconcat (gwgetdata->dir, 
 			                         str,
 						 NULL);
+	gwget_remember_downloads();
 }
 
 /* Return the gwgetdata that is selected in the treeview */
@@ -639,6 +660,8 @@ gwget_data_set_filename_from_url(GwgetData *gwgetdata,gchar *url)
 	} else {
 		gwgetdata->filename = g_strdup(filename);
 	}
+
+	gwget_remember_downloads();
 }
 
 /* Add a gwgetdata to the main window */
@@ -710,3 +733,76 @@ gwget_init_pref(Preferences *pref)
 	pref->docked   = FALSE;
 }
 
+static void
+gwget_gnotify_finished(GwgetData *gwgetdata) {
+   gchar *app_msg, *icon_msg, *fn_msg;
+   int sock;
+   char buf[8];
+
+   app_msg = g_strdup("APP:Download Finished");
+   icon_msg = g_strdup_printf("ICON:%s", gwgetdata->icon_name);
+   fn_msg = g_strdup_printf("MSG:%s", gwgetdata->filename);
+
+   sock = socket(AF_INET, SOCK_STREAM, 6);
+   if(sock > 0) {
+      struct sockaddr_in sa;
+
+      bzero(&sa, sizeof(sa));
+      sa.sin_family = AF_INET;
+      sa.sin_addr.s_addr = inet_addr("127.0.0.1");
+      sa.sin_port = htons(7206);
+      connect(sock, (struct sockaddr *)&sa, sizeof(sa));
+      send(sock, app_msg, strlen(app_msg) + 1, 0);
+      recv(sock, buf, 8, 0);
+      send(sock, icon_msg, strlen(icon_msg) + 1, 0);
+      recv(sock, buf, 8, 0);
+      send(sock, fn_msg, strlen(fn_msg) + 1, 0);
+      close(sock);
+   }
+
+   g_free(app_msg);
+   g_free(icon_msg);
+   g_free(fn_msg);
+	g_free(gwgetdata->icon_name);
+}
+
+void
+gwget_data_set_menus (GwgetData *gwgetdata)
+{
+
+
+	if (gwgetdata!=NULL) {
+		if (gwget_data_run(gwgetdata)) {
+			/* Menu items */
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),TRUE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),TRUE);
+			/* popup */
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),TRUE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),TRUE);
+		} else {
+			if (gwgetdata->state==DL_COMPLETED) {
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
+			} else {
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+			}
+		}
+	} else {
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"properties_menuitem"),FALSE);
+	}
+}
