@@ -16,6 +16,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#define _FILE_OFFSET_BITS 64
+
 #include <config.h>
 #include <gnome.h>
 #include <string.h>
@@ -26,10 +28,12 @@
 #include "gwget_data.h"
 #include "wget-log.h"
 #include "main_window.h"
+#include "systray.h"
+#include "utils.h"
 
 
 /* Function to convert the wget notation of file size (55,449,600) */ 
-static guint32 
+static guint64
 convert_wget_size (char *size)
 {
 	char *p = size;
@@ -42,7 +46,7 @@ convert_wget_size (char *size)
 		} else
 			p++;
    }
-   return atoi (size);
+   return atoll (size);
 }
 
 static void
@@ -173,10 +177,11 @@ wget_log_process_line (GwgetData *gwgetdata)
 		if (p != NULL) {
 			p += 8;
             /* Only set the total size if we don't have one yet. */
-            if (gwgetdata->total_size == 0) { 
+            if (gwgetdata->total_size == 0 && !gwgetdata->recursive) { 
             	gwget_data_set_total_size (gwgetdata,convert_wget_size (p));
 			}
 			gwget_data_set_state (gwgetdata, DL_RETRIEVING);
+			set_icon_newdownload();
 			/* Get session start time and session file start size */
 			if (stat (gwgetdata->local_filename, &file_stat) != -1) {
 		    	gwgetdata->session_start_time = file_stat.st_ctime;
@@ -232,9 +237,9 @@ wget_log_process_line (GwgetData *gwgetdata)
 					}
 					gwgetdata->session_elapsed = 0;
 				} else {
-                	/* We didn't get a length so, probably it's unspecified size
-                   so get the start of download by trying to catch the
-                   string "K ->" */
+		                	/* We didn't get a length so, probably it's unspecified size
+                			   so get the start of download by trying to catch the
+			                   string "K ->" */
 					p = strstr (gwgetdata->line, "K -> ");
 					if (p != NULL) {
 						/* Unspecified size, so set total_size to 0 */
@@ -249,20 +254,38 @@ wget_log_process_line (GwgetData *gwgetdata)
 					}
 					gwgetdata->session_elapsed = 0;
 				}
-			if (strstr (gwgetdata->line,"-- ") != NULL) {
-				gchar *tmp=NULL;
-				gint i,j;
-				j=0;
-				tmp = g_new(gchar,strlen(gwgetdata->line));
-				for (i=14;i<strlen(gwgetdata->line);i++) {
-					tmp[j]=gwgetdata->line[i];
-					j++;
+				if (strstr (gwgetdata->line,"-- ") != NULL) {
+					gchar *tmp=NULL;
+					gint i,j;
+					j=0;
+					tmp = g_new(gchar,strlen(gwgetdata->line));
+					for (i=14;i<strlen(gwgetdata->line);i++) {
+						tmp[j]=gwgetdata->line[i];
+						j++;
+					}
+					tmp[j]='\0';
+					gwget_data_set_filename_from_url(gwgetdata,tmp);
+					gwgetdata->local_filename = g_strconcat (gwgetdata->dir, gwgetdata->filename, NULL);
+				}
+
+				if (strstr (gwgetdata->line, "           =>") != NULL) {
+					gchar *tmp=NULL;
+					gint i,j;
+					j=0;
+					tmp = g_new(gchar,strlen(gwgetdata->line));
+					for (i=15;i<strlen(gwgetdata->line);i++) {
+						tmp[j]=gwgetdata->line[i];
+						j++;
+					}
+					/* Remove the las ' character */
+					tmp[j-1]='\0';
+					gwgetdata->local_filename = g_strdup (tmp);
+					gwgetdata->cur_size=0; 
+					gwgetdata->total_size=0; 
+					gwgetdata->session_start_time = 0;
+					gwgetdata->session_start_size = 0;
 				}
-				tmp[j]='\0';
-				gwget_data_set_filename_from_url(gwgetdata,tmp);
-				gwgetdata->local_filename = g_strconcat (gwgetdata->dir, gwgetdata->filename, NULL);
 			}
-		}
 		break;
 		default:
 		break;
@@ -270,7 +293,7 @@ wget_log_process_line (GwgetData *gwgetdata)
 	return 0;
 }
 
-void
+static void
 wget_log_read_log_line(GwgetData *gwgetdata) {
 	g_free(gwgetdata->line);
 	gwgetdata->line = NULL;
